//---------------------------------------------------------------------------
#ifndef pgpH
#define pgpH

#include "mixmaster.h"
//---------------------------------------------------------------------------
#define ARMORED 1
// key type
#define PGP_PUBKEY 6
#define PGP_SECKEY 5
#define PGP_PUBSUBKEY 14
#define PGP_SECSUBKEY 7
#define PGP_USERID 13
#define PGP_COMMENT 14
#define PGP_SESKEY 1
// sigs
#define PGP_SIG 2
// algorithms
#define PGP_ANY 0
#define PGP_ES_RSA 1
#define PGP_E_ELG 16
#define PGP_S_DSA 17
// SHA
#define SHA_DIGEST_LENGTH 20
// try to make the messages look similar to PGP 2.6.3i output
// compression is not always the same though
#define MIMIC

// packet type
#define PGP_ENCRYPTED 9

#define MSG_MAIL 1
#define PGP_PASS -2
#define PGP_ERR -1
#define PGP_SIGN 1
#define PGP_ENCRYPT 2
#define PGP_TEXT 16
#define PGP_REMAIL 8
#define PGP_CONVENTIONAL 4
#define PGP_NOARMOR 32

// error codes
#define PGP_OK 0	 	// valid message, not signed
#define PGP_SIGOK 1	  // valid signature
#define PGP_NOMSG 2		 // is not a OpenPGP message
#define PGP_NODATA 3	 // OpenPGP packet does not contain user data
#define PGP_SIGNKEY 4	 // can't verify signature
#define PGP_ERR -1		 // can't read message, no matching key found
#define PGP_PASS -2		 // bad passphrase
#define PGP_SIGBAD -3	 // bad signature

class TPgpUserId
{
public:
   char* UserId;
   TPgpUserId *Prev;
   TPgpUserId *Next;
   __fastcall TPgpUserId(TPgpUserId *prev, char *userid, TPgpUserId *next);
   __fastcall~TPgpUserId();
};

class TPgpKey : public TObject
{
private:
   BUFFER *key;
   AnsiString keyring;
   int GetType();
   AnsiString GetIdText8();
   byte GetVersion();
   long GetTimeStamp();
   int GetValidity();
   byte GetAlgorithm();
   int GetKeyBits();
   int GetKeyUserIds();
   int GetMPI(BUFFER *mpi);
   int PutMPI(BUFFER *buf, BUFFER *mpi);
   int KeyId(BUFFER *Keyid);
   AnsiString GetKeyring();
   bool GetIsPair();
   AnsiString __fastcall GetDescription();
public:
   TPgpKey *Next;
   TPgpKey *Prev;
   TPgpKey *SecKey; // used only if this is a pubkey
   TStringList *UserIds;
   __fastcall virtual TPgpKey(AnsiString &Keyring, BUFFER *Key);
   __fastcall virtual TPgpKey(BUFFER *Key);
   __fastcall ~TPgpKey();
   __property int Type = { read=GetType };
   __property AnsiString IdText8 = { read=GetIdText8 };
   __property byte Version = { read=GetVersion };
   __property long TimeStamp = { read=GetTimeStamp };
   __property int Validity = { read=GetValidity };
   __property byte Algorithm = { read=GetAlgorithm };
   __property int KeyBits = { read=GetKeyBits };
   __property AnsiString Keyring = { read=GetKeyring };
   __property bool IsPair = { read=GetIsPair };
   __property BUFFER *Key = { read=key };
   __property AnsiString Description = { read=GetDescription };
   long __fastcall Write(TFileStream *Stream);
   AnsiString __fastcall AsciiArmored();
};

class TPgpDb : public TObject
{
   KEYRING *pgpdb;
   TPgpKey *lastkey;
   TPgpKey *firstkey;
   int GetCount();
public:
   int error;
   virtual __fastcall TPgpDb(AnsiString PubRing, AnsiString SecRing);
   virtual __fastcall TPgpDb(TStrings *KeyRing);
   __fastcall ~TPgpDb();
   void __fastcall SaveToFile(AnsiString &FileName);
   void __fastcall SaveToStrings(TStrings *strings);
   void __fastcall SaveSecretToFile(AnsiString &Filename);
   TPgpKey* NextKey(TPgpKey *FromThis);
   TPgpKey* PrevKey(TPgpKey *FromThis);
   TPgpKey* KeyWithIdText8(AnsiString Text8);
   TPgpKey* GetKey(int pos);
   TPgpKey* FirstKey();
   TPgpKey* LastKey();
   int __fastcall AddKey(TPgpKey *Key);
   int __fastcall AppendDb(BUFFER *Key);
   int __fastcall Find(TPgpKey* Key);
   int __fastcall FindDescrip(AnsiString &Descrip);
   int __fastcall Delete(int pos);
   __property int KeyCount = { read=GetCount };
};
/*
class TPgpEncryptMsg : TObject
{
public:
   BUFFER *msg;
   __fastcall TPgpEncryptMsg(TStrings *AsciiBlock);
   __fastcall~TPgpEncryptMsg();
   bool __fastcall IsConventional();
   int __fastcall KeyId(BUFFER *keyid);
   int __fastcall Decrypt(AnsiString &SecringFile);
};
*/
int lock(FILE *f);
int unlock(FILE *f);
int unlockfile(LOCK *l);
LOCK *lockfile(char *filename);

KEYRING *pgpdb_openbuf(BUFFER *keys, BUFFER *encryptkey);
KEYRING *pgpdb_open(char *keyring, BUFFER *encryptkey, int writer);
KEYRING *pgpdb_new(char *keyring, int filetype, BUFFER *encryptkey);
int pgpdb_getnext(KEYRING *keydb, BUFFER *key);
int pgpdb_append(KEYRING *keydb, BUFFER *key);
int pgpdb_close(KEYRING *keydb);

int pgp_readkeyring(BUFFER *keys, char *filename);
int pgp_isconventional(BUFFER *b);
int pgp_packettype(BUFFER *b, long *len);
int pgp_ispacket(BUFFER *b);
int pgp_getpacket(BUFFER *in, BUFFER *p);
int pgp_packettype(BUFFER *b, long *len);
int pgp_dearmor(BUFFER *in, BUFFER *out);
int pgp_armor(BUFFER *in, int mode);
int pgp_keyid(BUFFER *key, BUFFER *keyid);
int pgp_packet(BUFFER *in, int type);
int mpi_get(BUFFER *b, BUFFER *mpi);
int pgp_nummpi(int algo);
int mpi_put(BUFFER *b, BUFFER *mpi);
int digestmem_sha1(byte *b, int n, BUFFER *md);
int digest_sha1(BUFFER *b, BUFFER *md);

// from 'simple' api code
void spgpKeyIDText8 (char *out, byte *keyid);
char* pgp_keyidtext8(BUFFER *keyId);

// my own efforts
int qpgp_keyringIds(AnsiString &KeyRing, TStringList *KeyIds);
int qpgp_keyringUserIds(AnsiString &KeyRing, TStringList *UserIds);
int pgp_keyUserIds(BUFFER *key, TStringList *UserIds, AnsiString *keyid=NULL);
int pgp_skipcert(BUFFER *key);

// SHA in libeay.dll
void __fastcall SHA1(byte *b, int n, byte* m);
typedef void (*_SHA1)(byte*, int, byte*);

extern TForm *ActMainForm;
#endif
